package rocketBugsRender;
/*
	ActinTracker... track actin density
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;

public class ActinTracker extends JFrame implements ChangeListener,ActionListener,KeyListener {
	static final int adBinCt = 100;	// number of bins
	static final double adExtent = 2.0;	// (m) distance from listeria center in which to track actin density
	static final double adTime = 10.0;	// (s) time from start to track
	static int histCt = 0;
	static HistogramPlus [] adArray = new HistogramPlus [100];	// array of histograms
	static HistogramPlus adHist = new HistogramPlus (adBinCt,-adExtent,adExtent,null,null,null);
	static HistoPlotter histPlot = new HistoPlotter();
	static DecimalFormat jpegIdFormat = new DecimalFormat ("#0000.#;#0000.#");
	
	static Listeria lis;
	
	public ActinTracker () {
		resetHist();
	
	}
	
	public static void setListeria (Listeria curLis) {
		lis = curLis;
	}
	
	public static void reportActin (A_Rod filament) {
		filament.mapActinLocation(adHist,lis);
	}
	
	public static void setHistTime (double curTime) {
		adHist.name = String.valueOf(Env.timeFormat.format(curTime));
	}
	
	public static void resetHist () {
		adHist = new HistogramPlus (adBinCt,-adExtent,adExtent,null,null,null);
		adHist.tossTail = true;
		adHist.tossHead = true;
		adHist.unitsString = "s";
	}
	
	public static void showHist (boolean scaleEach) {
		if (scaleEach) {
			histPlot.plotHist(adHist,true);
		} else {
			histPlot.plotHist(adHist,false);
		}
	}
	
	public static void addHistToArray () {
		if (histCt >= adArray.length) { growArray(); }
		adArray [histCt] = adHist;
		histCt++;
	}
	
	public static void growArray () {
		HistogramPlus [] tempArray = adArray;
		adArray = new HistogramPlus [histCt + 100];
		for (int i=0;i<histCt;i++) {
			adArray [i] = tempArray [i];
		}
	}
	
	public static void toFile () {
		if (histCt > 0) {
			// compact histogram array
			HistogramPlus [] tempArray = adArray;
			adArray = new HistogramPlus [histCt];
			for (int i=0;i<histCt;i++) { adArray[i] = tempArray[i]; }
			
			// get name to save
			String baseName = FileOps.getFileToSave("Save actin data to folder...",histPlot);
			if (baseName == null) { return; }
			
			
			// make folder
			String folderName = baseName;
			File mkFolder = new File(folderName);
			mkFolder.mkdir();
			
			// write to text file
			String textFileName = folderName + "/Hists_Text";
			FileOps.mkHistogramFile (adArray,textFileName);
			
			// write movie frames
			histPlot.arrayScale(adArray);	// prescale
			//histPlot.setYScale(7184);		// manually set Y scale for histogram of actin density... remove
			for (int i=0;i<histCt;i++) {
				talkln ("Writing histogram " + String.valueOf(i+1) + " of " + String.valueOf(histCt));
				String frameId = jpegIdFormat.format(i);
				String movieFrameName = folderName + "/frame" + frameId + ".png";
				histPlot.plotHist(adArray[i],false);
				//FileOps.takeRawPicture(histPlot.hPanel.theImage,movieFrameName);
				FileOps.pngFromBufferedImage(histPlot.hPanel.theImage,movieFrameName);
			}
			histCt = 0;
		}
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
	}
	
	public void keyReleased (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void stateChanged(ChangeEvent e) {   	
        		 
    } 
    
    private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.println(info);
	}
	
}
		