package rocketBugsRender;
/*
	Axis ... 
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import java.text.*;

public class Axis {
	DataColumn parentDataCol = null;
	PlotProps pProps = null;
	Color color;
	String name;
	Tic [] tics;
	int numTics = 7;
	int ticWidth = 5;
	int labelWidth = 0;		// number of pixels required to fit the label string
	int pixRange = 0;
	boolean showAxis = true;
	boolean showZero = true;
	boolean zeroMin = false;
	boolean fixedTic = true;
	double fixedTicInc = 0.0054;
	int yZero = 0;			// pixel value for zero
	DecimalFormat axisFormat = new DecimalFormat ("0.0#E0");
	DecimalFormat fixedTicFormat = new DecimalFormat ("0.0#E0");
	DecimalFormat timeFormat = new DecimalFormat ("0.000");
	DecimalFormat intFormat = new DecimalFormat ("0");
	DecimalFormat fixedTicNMFormat = new DecimalFormat ("0.0");
	DecimalFormat defaultFormat = new DecimalFormat ("0.0#E0");
	
	public Axis (DataColumn col) {
		parentDataCol = col;
		pProps = col.pProps;
	}
	
	/*public Axis (DataColumn col, String name) {
		parentDataCol = col;
		this.name = name;
	}*/
	
	
	public Tic [] getTics () {
		return tics;
	}
	
	public void makeTics (double minVal, double range, int minPix, int maxPix) {
		pixRange = maxPix - minPix;
		setZeroPix(minVal,range,minPix,maxPix);
		if (fixedTic) {
			makeFixedTics(minVal,range,minPix,maxPix);
		} else {
			makeValueTics(minVal,range,minPix,maxPix);
		}
	}
	
	private void setZeroPix(double minVal, double range, int minPix, int maxPix) {
		if (minVal < 0) {
			double negFrac = -minVal/range;
			yZero = (int)((maxPix-minPix)*negFrac) + minPix;
		}
	}
	
	private void makeFixedTics (double minVal, double range, int minPix, int maxPix) {
		double fixedTicRatio = (range)/fixedTicInc;
		double valToPix = (double)((maxPix-minPix))/(range);
		int numFixedTics = (int)(Math.floor(fixedTicRatio)) + 1;
		int ticPixInc = (int)(fixedTicInc*valToPix);
		tics = new Tic [numFixedTics];
		for (int i=0;i<numFixedTics;i++) {
			int ticPix = minPix + i*ticPixInc;
			//String labelString = "" + String.valueOf(fixedTicFormat.format(fixedTicInc))+ "m  ";
			String labelString = " " + String.valueOf(fixedTicNMFormat.format(fixedTicInc*1000))+ "nm  ";
			tics[i] = new Tic (ticPix, ticWidth, labelString);
		}
		if (numFixedTics > 15) { makeValueTics (minVal, range, minPix, maxPix); }
	}
	
	private void makeValueTics (double minVal, double range, int minPix, int maxPix) {
		double ticInc = (range)/(double)(numTics-1);
		int ticPixInc = (maxPix-minPix)/(numTics-1);
		tics = new Tic [numTics];
		int longestLabelLength = 0;
		for (int i=0;i<numTics;i++) {
			int ticPix = minPix + i*ticPixInc;
			double labelVal = pProps.unitMult*(minVal + i*ticInc);
			String labelString = String.valueOf(pProps.textFormat.format(labelVal));
			if (name.equals("time")) { labelString = String.valueOf(timeFormat.format(labelVal) + " s"); }
			tics[i] = new Tic (ticPix, ticWidth, labelString);
			if (labelString.length() > longestLabelLength) { labelWidth = 9*labelString.length()+12; } // set labelWidth by longest string
		}
		
		
	}
	
	public void turnOff() {
		showAxis = false;
	}
	
	public void turnOn() {
		showAxis = true;
	}
	
	public void copyVals (Axis cpA) {
		tics = cpA.tics;
	}
			
}