package rocketBugsRender;
/*
	ButtonList ... a 1D column of AbstractButtons
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;	

public class ButtonList {
	AbstractButton [] list;
	JLabel titleLabel = new JLabel();
	String title;
	String type;
	int listCt;
	int index = -1;		// keep track of location in ButtonTables
	Box lBox = new Box(BoxLayout.Y_AXIS);
	boolean init = false; 	// initialization flag
	Color backColor = Color.black;
	Font font = Env.listFont;
	
	public ButtonList (String title, String [] names, String type, int index) {
		this.title = title;
		this.titleLabel.setText(title);
		this.type = type;
		this.index = index;
		this.listCt = names.length;
		if (type.equals("JCheckBox")) { checkBoxInit(names); }
		if (type.equals("JButton")) { buttonInit(names); }
		if (type.equals("JRadioButton")) { radioInit(names); }
		if (!init) { talkln ("Initialization of ButtonList with " + type + " failed"); }
		updateLayout();
	}
	
	public void addNewButton (String name) {
		if (type.equals("JCheckBox")) { addCheckBox(name); }
		if (type.equals("JButton")) { addButton(name); }
		if (type.equals("JRadioButton")) { addRadio(name); }
		updateLayout();
	}
	
	public void addCheckBox (String name) {
		JCheckBox [] tempList = (JCheckBox[])list;
		list = new JCheckBox [listCt+1];
		for (int i=0;i<listCt;i++) {
			list[i] = tempList[i];
		}
		list[listCt] = new JCheckBox(name);
		list[listCt].setFont(font);
		listCt++;
	}
	
	public void addButton (String name) {
		JButton [] tempList = (JButton[])list;
		list = new JButton [listCt+1];
		for (int i=0;i<listCt;i++) {
			list[i] = tempList[i];
		}
		list[listCt] = new JButton(name);
		list[listCt].setFont(font);
		listCt++;
	}
	
	public void addRadio (String name) {
		JRadioButton [] tempList = (JRadioButton[])list;
		list = new JRadioButton [listCt+1];
		for (int i=0;i<listCt;i++) {
			list[i] = tempList[i];
		}
		list[listCt] = new JRadioButton(name);
		list[listCt].setFont(font);
		list[listCt].setBackground(backColor); 
		list[listCt].setForeground(Color.white);
		listCt++;
	}
	
	private void checkBoxInit(String [] names) {
		list = new JCheckBox[listCt];
		for (int i=0;i<listCt;i++) { 
			list[i] = new JCheckBox(names[i]); 
			list[i].setFont(font); 
		}
		init = true;
	}
	
	private void buttonInit(String [] names) {
		list = new JButton[listCt];
		for (int i=0;i<listCt;i++) { 
			list[i] = new JButton(names[i]);
			list[i].setFont(font); 
		}
		init = true;
	}
	
	private void radioInit(String [] names) {
		list = new JRadioButton[listCt];
		for (int i=0;i<listCt;i++) { 
			list[i] = new JRadioButton(names[i]);
			list[i].setFont(font); 
			list[i].setBackground(backColor); 
			list[i].setForeground(Color.white);
		}
		titleLabel.setBackground(backColor);
		titleLabel.setForeground(Color.white);
		init = true;
	}
	
	public void updateColors(Color newColor) {
		backColor = newColor;
		titleLabel.setBackground(backColor);
		if (backColor.equals(Color.white)) { 
			titleLabel.setForeground(Color.black);
		} else {
			titleLabel.setForeground(Color.white);
		}
		for (int i=0;i<listCt;i++) {
			list[i].setBackground(backColor);
		}
	}
	
	
	public void updateLayout() {
		// set preferred button sizes
		Dimension prefSize;
		for (int i=0;i<listCt;i++) {
			prefSize = list[i].getPreferredSize();
			prefSize.height = 14;
			list[i].setPreferredSize(prefSize);
		}
		lBox.removeAll();
		lBox.add(titleLabel);
		for (int i=0;i<listCt;i++) {
			lBox.add(list[i]);
		}
	}
	
	public void setActionCommand(String actionC, ActionListener listener) {
		if (actionC != null) {
			for (int i=0;i<listCt;i++) {
				list[i].setActionCommand(actionC);
				list[i].addActionListener(listener);
			}
		}
	}
	
	public void addToButtonGroup(ButtonGroup bGrp) {
		for (int i=0;i<listCt;i++) {
			bGrp.add(list[i]);
		}
	}
	
	public void rmFromButtonGroup(ButtonGroup bGrp) {
		for (int i=0;i<listCt;i++) {
			bGrp.remove(list[i]);
		}
	}
	
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}

}