package rocketBugsRender;
/*
	FileOps.... file that handles the i/o
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.imageio.*;

public class FileOps {
	static String sepString = ";";	// the character used for separating data in text files
	static String sepString2 = ",";	// another character used for separating data in text files
	static String lastFileActinReported = null;
	
	// definitions used in file reading and writing
	static final int ROD = 1;
	static final int NODE = 2;
	static final int BLOB = 3;
	static final int LISTERIA = 4;
	static final int LISTERIA_SIZED = 5;
	
	public static void pngFromBufferedImage(BufferedImage bImg, String fileName) {
	    try {
		  // create file
	      File outFile = new File(fileName);
		  RenderedImage rendImage = (RenderedImage)bImg;
		  ImageIO.write(rendImage, "png", outFile);
	    }
	    catch( Exception e )
	    {
	      e.printStackTrace();
	    }
	 }
	
	public static String getFileToSave (String dialogTitle, JFrame parent) {
		File file = null;
		String fileName = null;
		JFileChooser saveTo = new JFileChooser();
		saveTo.setDialogTitle(dialogTitle);
		int fileQuery = saveTo.showSaveDialog(parent);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            file = saveTo.getSelectedFile();
            fileName = file.getAbsolutePath();
            talkln (dialogTitle + fileName);
        } else {
            talkln ("Save cancelled by user");
            return null;
        }
        return fileName;
	}
	
	public void paintQkFrameFromFile (String fileName, RBRPanel thePanel, boolean listeriaOnly) {
		qkFrameFromFile (fileName, thePanel, listeriaOnly, false);
	}
	
	public void loadQkFrameFromFile (String fileName, RBRPanel thePanel, boolean listeriaOnly) {
		qkFrameFromFile (fileName, thePanel, listeriaOnly, true);
	}
	
	public static int [] getDataDims (String fileName) {
		int colCt = 0;
		int rowCt = 0;
		File dataFile = new File (fileName);
		if ( ! dataFile.exists() || ! dataFile.canRead() ) {
			talkln ("Can't find data file... bigtime error");
		} else {
			try {
				FileReader fileR = new FileReader(dataFile);
				BufferedReader in = new BufferedReader(fileR);
				String titleLine = in.readLine(); // run title line
				String curLine = in.readLine(); // column titles
				// count columns
				int nextIndex = 0;
				colCt = -1;		// so we don't have to subtract the last increment
				while (nextIndex != -1) {
					nextIndex = curLine.indexOf(sepString,nextIndex+1);
					colCt++;
				}
				// count lines
				rowCt = -1;
				while (curLine != null) {
					curLine = in.readLine();
					rowCt++;
				}
					
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
		return new int [] {colCt,rowCt,2};
	}
	
	public static void getDataNames (String fileName, int nameCt, DataColumn [] dataVals) {
		File dataFile = new File (fileName);
		if ( ! dataFile.exists() || ! dataFile.canRead() ) {
			talkln ("Can't find data file... bigtime error");
		} else {
			try {
				FileReader fileR = new FileReader(dataFile);
				BufferedReader in = new BufferedReader(fileR);
				String titleLine = in.readLine(); // run title line
				talkln (titleLine);
				String firstLine = in.readLine(); // column titles
				int curIndex = 0;
				int nextIndex = 0;
				int j=0;
				while ((nextIndex != -1) & (curIndex < firstLine.length())) {
					nextIndex = firstLine.indexOf(sepString,curIndex);
					if (nextIndex == -1) { 
						dataVals[j].setName(firstLine.substring(curIndex,firstLine.length()));
					} else {
						dataVals[j].setName(firstLine.substring(curIndex,nextIndex));
					}
					j++;
					curIndex = nextIndex+1;
				}
				
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
	}
	
	public static String [] getDataNamesOnly (String fileName) {
		String [] names = null;
		File dataFile = new File (fileName);
		if ( ! dataFile.exists() || ! dataFile.canRead() ) {
			talkln ("Can't find data file... bigtime error");
		} else {
			try {
				FileReader fileR = new FileReader(dataFile);
				BufferedReader in = new BufferedReader(fileR);
				String titleLine = in.readLine(); // run title line
				String firstLine = in.readLine(); // column titles
				// count columns
				int nextIndex = 0;
				int colCt = -1;		// so we don't have to subtract the last increment
				while (nextIndex != -1) {
					nextIndex = firstLine.indexOf(sepString,nextIndex+1);
					colCt++;
				}
				names = new String [colCt];
				int curIndex = 0;
				nextIndex = 0;
				int j=0;
				while ((nextIndex != -1) & (curIndex < firstLine.length())) {
					nextIndex = firstLine.indexOf(sepString,curIndex);
					if (nextIndex == -1) { 
						names[j] = firstLine.substring(curIndex,firstLine.length());
					} else {
						names[j] = firstLine.substring(curIndex,nextIndex);
					}
					j++;
					curIndex = nextIndex+1;
				}
				
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
		return names;
	}
	
	public static void getDataVals (String fileName, int [] dims, DataColumn [] dataVals) {
		String curLine;
		String curVal;
		Double curDub;
		File dataFile = new File (fileName);
		if ( ! dataFile.exists() || ! dataFile.canRead() ) {
			talkln ("Can't find data file... bigtime error");
		} else {
			try {
				FileReader fileR = new FileReader(dataFile);
				BufferedReader in = new BufferedReader(fileR);
				for (int k=0; k<dims[2]; k++) { curLine = in.readLine(); }	// pass title and name lines
				// loop for all double vals
				for (int i=0; i<dims[1]; i++) { //loop through all data lines
					curLine = in.readLine();
					//talkln ("Line #" + String.valueOf(i) + " : " + curLine);
					int curIndex = 0;
					int nextIndex = 0;
					for (int j=0;j<dims[0];j++) {
						nextIndex = curLine.indexOf(sepString,curIndex);
						curVal = curLine.substring(curIndex,nextIndex);
						// change string to double... store value in array
						if (curVal.equals("NaN")) {
							if (Env.dataColumnsOn[j]) { dataVals[j].data[i] = Double.NaN; }
						} else {
							curDub = Double.valueOf(curVal);
							if (Env.dataColumnsOn[j]) { dataVals[j].data[i] = curDub.doubleValue(); }
						}
						// increment index
						curIndex = nextIndex+1;
					}
				}
				
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
	}
	
	public static void getCheckedDataVals (String fileName, int [] dims, DataColumn [] dataVals) {
		String curLine;
		String curVal;
		Double curDub;
		File dataFile = new File (fileName);
		if ( ! dataFile.exists() || ! dataFile.canRead() ) {
			talkln ("Can't find data file... bigtime error");
		} else {
			try {
				FileReader fileR = new FileReader(dataFile);
				BufferedReader in = new BufferedReader(fileR);
				for (int k=0; k<dims[2]; k++) { curLine = in.readLine(); }	// pass title and name lines
				// loop for all double vals
				for (int i=0; i<dims[1]; i++) { //loop through all data lines
					curLine = in.readLine();
					//talkln ("Line #" + String.valueOf(i) + " : " + curLine);
					int curIndex = 0;
					int nextIndex = 0;
					for (int j=0;j<dims[0];j++) {
						nextIndex = curLine.indexOf(sepString,curIndex);
						curVal = curLine.substring(curIndex,nextIndex);
						// change string to double... store value in array
						if (curVal.equals("NaN")) {
							if (Env.dataColumnsOn[j]) { dataVals[j].data[i] = Double.NaN; }
						} else {
							curDub = Double.valueOf(curVal);
							if (Env.dataColumnsOn[j]) { dataVals[j].data[i] = curDub.doubleValue(); }
						}
						// increment index
						curIndex = nextIndex+1;
					}
				}
				
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
	}
	
	/*public static void getDataVals (String fileName, int [] dims, DataColumn [] dataVals) {
		String curLine;
		String curVal;
		Double curDub;
		File dataFile = new File (fileName);
		if ( ! dataFile.exists() || ! dataFile.canRead() ) {
			talkln ("Can't find data file... bigtime error");
		} else {
			try {
				FileReader fileR = new FileReader(dataFile);
				BufferedReader in = new BufferedReader(fileR);
				for (int k=0; k<dims[2]; k++) { curLine = in.readLine(); }	// pass title and name lines
				// loop for all double vals
				for (int i=0; i<dims[1]; i++) { //loop through all data lines
					curLine = in.readLine();
					int curIndex = 0;
					int nextIndex = 0;
					int j=0;
					while ((nextIndex != -1) & (curIndex < curLine.length())) {
						nextIndex = curLine.indexOf(sepString,curIndex);
						if (nextIndex == -1) { 
							curVal = curLine.substring(curIndex,curLine.length());
						} else {
							curVal = curLine.substring(curIndex,nextIndex);
						}
						// change string to double... store value in array
						if (curVal.equals("NaN")) {
							if (Env.dataColumnsOn[j]) { dataVals[j].data[i] = Double.NaN; }
						} else {
							curDub = Double.valueOf(curVal);
							if (Env.dataColumnsOn[j]) { dataVals[j].data[i] = curDub.doubleValue(); }
						}
						// increment position on line
						j++;
						curIndex = nextIndex+1;
					}
				}
				
			} catch (IOException ioe) { talkln ("An error trying to save environment file"); }
		}
	}*/
	
	public void qkFrameFromFile (String fileName, RBRPanel thePanel, boolean listeriaOnly, boolean loadFile) {
		// integer definitions used in file writing and reading
		A_Rod staticRod = new A_Rod ();		// the one rod for loading from file
		Listeria staticLis = new Listeria ();		// the one Listeria for loading from file
		int actACount = 0;
		Color playerColor = Color.black;
		Pt3D end1, end2, coord, angle,lisShape;
		Pt3D tetherPt = new Pt3D();
		Pt3D [] actALoc;
		int noPs;	// number of players in file... the first int written
		int i1,i2,i3;	// integers
		int playerType;  	// type of player  1=A_Rod, 2=A_Node, 3=A_Blob, 4=Listeria, 5=Listeria with size info
		byte plusCapped, minusCapped;	// boolean (1 = true, 0 = false) flags for capping/tethering of filaments
		float p1,p2,p3,p4,p5; 	// parameters that define a player... they vary depending on type.
		try {
			FileInputStream fis = new FileInputStream (fileName);
			DataInputStream ds = new DataInputStream (fis);
			noPs = ds.readInt();
			if (noPs == -1) {	// then this file is giving us the deltaT for the sim
				Env.deltaT = ds.readFloat();
				noPs = ds.readInt();	// this would be the number of players
			}
			if (loadFile) { Player.playerCount = 0; }
			for (int j = 0; j < noPs; j++) {
				playerType = (int)ds.readByte();
				switch (playerType) {
					case ROD: 	// A_Rod
						plusCapped = ds.readByte();
						minusCapped = ds.readByte();
						/*actACount = ds.readInt();
						actALoc = new Pt3D[actACount];
						for (int k=0; k<actACount; k++) {
							actALoc[k] = readPt3D(ds);
						}*/
						end1 = readPt3D(ds);
						end2 = readPt3D(ds);
						angle = readPt3D(ds);
						i1 = ds.readInt();	// number of actin monomers
						i2 = ds.readInt();	// number of ADP-Pi monomers
						i3 = ds.readInt();	// number of ADP monomers
						p1 = ds.readFloat();	// helix angle
						if (! listeriaOnly) {
							if (loadFile) {
								Player.playerList[Player.playerCount] = A_Rod.InitializeFromFileInfo(end1,end2,angle,i1,i2,i3,p1, plusCapped, minusCapped);
								Player.playerCount ++;
							} else {
								staticRod.initializeFromFileInfo(end1,end2,angle,i1,i2,i3,p1, plusCapped, minusCapped);
								thePanel.drawPlayer(staticRod); 
								if (Env.trackActin) { ActinTracker.reportActin(staticRod); }
							}
						}
						break;
					case NODE:	// A_Node
						p1 = ds.readFloat();
						p2 = ds.readFloat();
						p3 = ds.readFloat();
						p4 = ds.readFloat();
						//thePanel.drawNode(p1,p2,p3,p4);
						break;
					case BLOB:	// A_Blob
						p1 = ds.readFloat();
						p2 = ds.readFloat();
						p3 = ds.readFloat();
						//thePanel.drawBlob(p1,p2,p3);
						break;
					case LISTERIA:	// Listeria
						end1 = readPt3D(ds);	// rec1 for listeria
						end2 = readPt3D(ds);	// rec2 for listeria
						angle = readPt3D(ds);
						Env.curLisPos.copy(end1);	// save end1 as current Listeria position
						if (loadFile) {
							Player.playerList [Player.playerCount] = Listeria.InitializeFromFileInfo (end1,end2,angle);
							Player.playerCount ++;
						} else {
							staticLis.initializeFromFileInfo(end1,end2,angle);
							if (Env.trackActin) { ActinTracker.setListeria(staticLis); }
							if (Env.trackListeria) { 
								Pt3D zoomCen = Pt3D.Scale(-Listeria.capRad, staticLis.uVec);	// vec from end1 to tip of listeria
								zoomCen.inc(end1);	// add end1 coordinate.. this is new zoomCen
								thePanel.newZoomCenter(zoomCen); 
							}
							if (listeriaOnly) { j = noPs; } // got the listeria....hop on outta here
							thePanel.drawPlayer(staticLis);
						}
						break;
					case LISTERIA_SIZED:	// Listeria with size info
						double lisLength = (double)ds.readFloat();
						double lisCylDiam = (double)ds.readFloat();
						end1 = readPt3D(ds);	// rec1 for listeria
						end2 = readPt3D(ds);	// rec2 for listeria
						angle = readPt3D(ds);
						Env.curLisPos.copy(end1);	// save end1 as current Listeria position
						if (loadFile) {
							Player.playerList [Player.playerCount] = Listeria.InitializeFromFileInfo (lisLength,lisCylDiam,end1,end2,angle);
							Player.playerCount ++;
						} else {
							staticLis.initializeFromFileInfo(lisLength,lisCylDiam,end1,end2,angle);
							if (Env.trackActin) { ActinTracker.setListeria(staticLis); }
							if (Env.trackListeria) { 
								Pt3D zoomCen = Pt3D.Scale(-Listeria.capRad, staticLis.uVec);	// vec from end1 to tip of listeria
								zoomCen.inc(end1);	// add end1 coordinate.. this is new zoomCen
								thePanel.newZoomCenter(zoomCen); 
							}
							if (listeriaOnly) { j = noPs; } // got the listeria....hop on outta here
							thePanel.drawPlayer(staticLis);
						}
						break;
				}
			} 
			if (loadFile) { Env.curFileLoaded = true; }
		}  catch (IOException ioe) { talkln ("some sort of error reading binary file"); }
		if (Env.reportNumTethered) { talkln ("# of ActAs = " + String.valueOf(actACount)); }
		if ((Env.trackActin) & (!loadFile) & (fileName != lastFileActinReported)) {
			ActinTracker.setHistTime(Env.curTime);
			ActinTracker.addHistToArray();
			ActinTracker.showHist(true);
			ActinTracker.resetHist();
			lastFileActinReported = fileName;
		}
	}
	
	
	/*public void qkFrameFromFile (String fileName, ZoomPanel thePanel, boolean listeriaOnly, boolean loadFile) {
		A_Rod staticRod = new A_Rod ();		// the one rod for loading from file
		Listeria staticLis = new Listeria ();		// the one Listeria for loading from file
		int actACount = 0;
		Color playerColor = Color.black;
		Pt3D end1, end2, coord, angle,lisShape;
		Pt3D tetherPt = new Pt3D();
		Pt3D [] actALoc;
		int noPs;	// number of players in file... the first int written
		int i1,i2,i3;	// integers
		byte pType;  	// type of player  1=A_Rod, 2=A_Node, 3=A_Blob, 4=Listeria
		byte plusCapped, minusCapped;	// boolean (1 = true, 0 = false) flags for capping/tethering of filaments
		float p1,p2,p3,p4,p5; 	// parameters that define a player... they vary depending on type.
		try {
			FileInputStream fis = new FileInputStream (fileName);
			DataInputStream ds = new DataInputStream (fis);
			noPs = ds.readInt();
			if (noPs == -1) {	// then this file is giving us the deltaT for the sim
				Env.deltaT = ds.readFloat();
				noPs = ds.readInt();	// this would be the number of players
			}
			if (loadFile) { Player.playerCount = 0; }
			for (int j = 0; j < noPs; j++) {
				pType = ds.readByte();
				if (pType == 1) {	// A_Rod
					plusCapped = ds.readByte();
					minusCapped = ds.readByte();
					//actACount = ds.readInt();
					//actALoc = new Pt3D[actACount];
					//for (int k=0; k<actACount; k++) {
					//	actALoc[k] = readPt3D(ds);
					//}
					end1 = readPt3D(ds);
					end2 = readPt3D(ds);
					angle = readPt3D(ds);
					i1 = ds.readInt();	// number of actin monomers
					i2 = ds.readInt();	// number of ADP-Pi monomers
					i3 = ds.readInt();	// number of ADP monomers
					p1 = ds.readFloat();	// helix angle
					if (! listeriaOnly) {
						if (loadFile) {
							Player.playerList[Player.playerCount] = A_Rod.InitializeFromFileInfo(end1,end2,angle,i1,i2,i3,p1, plusCapped, minusCapped);
							Player.playerCount ++;
						} else {
							staticRod.initializeFromFileInfo(end1,end2,angle,i1,i2,i3,p1, plusCapped, minusCapped);
							thePanel.drawPlayer(staticRod); 
							if (Env.trackActin) { ActinTracker.reportActin(staticRod); }
						}
					}
				} else {
					if (pType == 2) {	// A_Node
						p1 = ds.readFloat();
						p2 = ds.readFloat();
						p3 = ds.readFloat();
						p4 = ds.readFloat();
						//thePanel.drawNode(p1,p2,p3,p4);
					} else {
						if (pType == 3) { 	// A_Blob
							p1 = ds.readFloat();
							p2 = ds.readFloat();
							p3 = ds.readFloat();
							//thePanel.drawBlob(p1,p2,p3);
						} else {
							if (pType == 4) {	// Listeria
								end1 = readPt3D(ds);	// rec1 for listeria
								end2 = readPt3D(ds);	// rec2 for listeria
								angle = readPt3D(ds);
								Env.curLisPos.copy(end1);	// save end1 as current Listeria position
								if (loadFile) {
									Player.playerList [Player.playerCount] = Listeria.InitializeFromFileInfo (end1,end2,angle);
									Player.playerCount ++;
								} else {
									staticLis.initializeFromFileInfo(end1,end2,angle);
									if (Env.trackActin) { ActinTracker.setListeria(staticLis); }
									if (Env.trackListeria) { 
										Pt3D zoomCen = Pt3D.Scale(-Listeria.capRad, staticLis.uVec);	// vec from end1 to tip of listeria
										zoomCen.inc(end1);	// add end1 coordinate.. this is new zoomCen
										thePanel.newZoomCenter(zoomCen); 
									}
									if (listeriaOnly) { j = noPs; } // got the listeria....hop on outta here
									thePanel.drawPlayer(staticLis);
								}
							}
						}
					}
				}
			} 
			if (loadFile) { Env.curFileLoaded = true; }
		}  catch (IOException ioe) { talkln ("some sort of error reading binary file"); }
		if (Env.reportNumTethered) { talkln ("# of ActAs = " + String.valueOf(actACount)); }
		if ((Env.trackActin) & (!loadFile) & (fileName != lastFileActinReported)) {
			ActinTracker.setHistTime(Env.curTime);
			ActinTracker.addHistToArray();
			ActinTracker.showHist(true);
			ActinTracker.resetHist();
			lastFileActinReported = fileName;
		}
	} */
		
	
	private Pt3D readPt3D (DataInputStream ds) {
	 	try {
			float f1 = ds.readFloat();
			float f2 = ds.readFloat();
			float f3 = ds.readFloat();
			return new Pt3D(f1,f2,f3);
		} catch (IOException ioe) {}
		return null;
	}
	
	/*public void takeRawPicture (String fileName, int frameNum) {
		FileOutputStream fos;
		DataOutputStream ds;
		byte alpha,red,green,blue;
		int imWidth = ZoomPanel.winSize;
		int imHeight = ZoomPanel.winSize;
		int [] pxls = new int [imWidth*imHeight];
		byte [] rowOfBytes = new byte [imWidth*4];
		PixelGrabber pg = new PixelGrabber(ZoomPanel.theImage,0,0,imWidth,imHeight,pxls,0,imWidth);
		try {
            		pg.grabPixels();
      		} catch (InterruptedException e) {
          		talkln ("interrupted waiting for pixels!");
           		return;
        	}
        	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
          	 	talkln ("image fetch aborted or errored");
            		return;
       	 	}
  
		try {
			fos = new FileOutputStream (fileName);
			ds = new DataOutputStream (fos);
			for (int j = 0; j < imHeight; j++) {
				for (int i = 0; i < imWidth; i++) {
					alpha =  (byte) ((pxls[j*imWidth+i] >> 24) & 0xff);
					red = (byte) ((pxls[j*imWidth+i] >> 16) & 0xff);
					green = (byte) ((pxls[j*imWidth+i] >> 8) & 0xff);
					blue =  (byte) ((pxls[j*imWidth+i]        ) & 0xff);
					rowOfBytes[4*i] = alpha;
					rowOfBytes[4*i+1] = red;
					rowOfBytes[4*i+2] = green;
					rowOfBytes[4*i+3] = blue;
				}
				ds.write(rowOfBytes,0,4*imWidth);
			}
		} catch (IOException ioe) { talkln ("some sort of error writing binary file from frame" + String.valueOf(frameNum)); }
	}*/
	
	public static void takeRawPicture (Image theImage, String fileName) {
		FileOutputStream fos;
		DataOutputStream ds;
		byte alpha,red,green,blue;
		int imWidth = theImage.getWidth(null);
		int imHeight = theImage.getHeight(null);
		int [] pxls = new int [imWidth*imHeight];
		byte [] rowOfBytes = new byte [imWidth*4];
		PixelGrabber pg = new PixelGrabber(theImage,0,0,imWidth,imHeight,pxls,0,imWidth);
		try {
            		pg.grabPixels();
      		} catch (InterruptedException e) {
          		talkln ("interrupted waiting for pixels!");
           		return;
        	}
        	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
          	 	talkln ("image fetch aborted or errored");
            		return;
       	 	}
  
		try {
			fos = new FileOutputStream (fileName);
			ds = new DataOutputStream (fos);
			for (int j = 0; j < imHeight; j++) {
				for (int i = 0; i < imWidth; i++) {
					alpha =  (byte) ((pxls[j*imWidth+i] >> 24) & 0xff);
					red = (byte) ((pxls[j*imWidth+i] >> 16) & 0xff);
					green = (byte) ((pxls[j*imWidth+i] >> 8) & 0xff);
					blue =  (byte) ((pxls[j*imWidth+i]        ) & 0xff);
					rowOfBytes[4*i] = alpha;
					rowOfBytes[4*i+1] = red;
					rowOfBytes[4*i+2] = green;
					rowOfBytes[4*i+3] = blue;
				}
				ds.write(rowOfBytes,0,4*imWidth);
			}
		} catch (IOException ioe) { talkln ("some sort of error writing binary file from frame"); }
	}
	
	public static void mkHistogramFile (Histogram hist) {
		try { 
			//if (hist.path == null) { hist.path = "~/"; }
			File histoFile = new File (hist.path + "/" + hist.name);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			histoPW.println("interval; count = " + String.valueOf(hist.ptCt));
			for (int i=0; i<hist.ctVals.length; i++) {
				histoPW.println(String.valueOf(hist.intervalVals[i]) + ";" + String.valueOf(hist.ctVals[i]));
			}
			talkln ("wrote histogram file with " + String.valueOf(hist.ptCt) + " pts");
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	
	public static void mkHistogramFile (HistogramPlus hist) {
		try { 
			String fileName = getFileToSave("Save histogram of " + hist.name + " to ...", hist.parent);
			if (fileName == null ) { return; }
			File histoFile = new File (fileName);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			// column headings
			hist.printIntervalTitle(histoPW,sepString); 
			hist.printBinTitles(histoPW,sepString);
			histoPW.println("");
			// data
			for (int i=0;i<hist.binCt;i++) {
				hist.printInterval(histoPW,sepString,i);
				hist.printBinSum(histoPW,sepString,i); 
				if (i<hist.infoS.length) { histoPW.print(hist.infoS[i]); }
				histoPW.println("");
			}
			talk ("wrote histogram file with " + String.valueOf(hist.ptCt) + " pts");
			if (hist.reportAve) { talk (" and " + String.valueOf(hist.passCt) + " passes"); }
			talkln ("");
			
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	
	/*public static void mkHistogramFile (HistogramPlus [] hists) {
		try { 
			String fileName = getFileToSave("Save Histogram to ...", hists[0].parent);
			if (fileName == null ) { return; }
			File histoFile = new File (fileName);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			// column headings
			histoPW.print("interval");
			for (int k=0;k<hists.length;k++) {
				histoPW.print(";" + hists[k].name + " ct=" + String.valueOf(hists[k].ptCt));
				if (hists[k].reportNorm) { histoPW.print(";" + hists[k].name + " norm"); }
				if (hists[k].reportAve) { histoPW.print(";" + hists[k].name + " ave passCt=" + String.valueOf(hists[k].passCt)); }
			}
			histoPW.println("");
			// data
			for (int i=0;i<hists[0].binCt;i++) {
				histoPW.print(String.valueOf(hists[0].getInterval(i)));
				for (int k=0;k<hists.length;k++) {
					histoPW.print(";" + String.valueOf(hists[k].getCtVal(i)));
					if (hists[k].reportNorm) { histoPW.print(";" + String.valueOf(hists[k].getNormCtVal(i))); }
					if (hists[k].reportAve) { histoPW.print(";" + String.valueOf(hists[k].getAveCtVal(i))); }
				}
				histoPW.println("");
			}
			talk ("wrote file for " + String.valueOf(hists.length) + " histograms with " + String.valueOf(hists[0].ptCt) + " pts");
		 	talk (" and " + String.valueOf(hists[0].passCt) + " passes"); 
			talkln ("");
			
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}*/
	
	public static void mkHistogramFile (HistogramPlus [] hists, String fileName) {	// method if filename already known
		//HistogramPlus.setScaledNormMag(hists);		// set scaling for histogram output
		try { 
			File histoFile = new File (fileName);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			// column headings
			for (int k=0;k<hists.length;k++) {
				if (k==0) { hists[k].printIntervalTitle(histoPW,sepString); }
				hists[k].printBinTitles(histoPW,sepString);
			}
			histoPW.println("");
			// data
			for (int i=0;i<hists[0].binCt;i++) {
				hists[0].printInterval(histoPW,sepString,i);
				for (int k=0;k<hists.length;k++) { hists[k].printBinSum(histoPW,sepString,i); }
				//if (i<hists[0].infoS.length) { histoPW.print(hists[0].infoS[i]); }
				histoPW.println("");
			}
			talk ("wrote file for " + String.valueOf(hists.length) + " histograms with " + String.valueOf(hists[0].ptCt) + " pts");
		 	talk (" and " + String.valueOf(hists[0].passCt) + " passes"); 
			talkln ("");
			
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	
	
	public static void mkHistogramFile (HistogramPlus [] hists) {
		HistogramPlus.setScaledNormMag(hists);		// set scaling for histogram output
		try { 
			String fileName = getFileToSave("Save histogram of " + hists[0].name + " to ...", hists[0].parent);
			if (fileName == null ) { return; }
			File histoFile = new File (fileName);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			// column headings
			for (int k=0;k<hists.length;k++) {
				if (k==0) { hists[k].printIntervalTitle(histoPW,sepString); }
				hists[k].printBinTitles(histoPW,sepString);
			}
			histoPW.println("");
			// data
			for (int i=0;i<hists[0].binCt;i++) {
				hists[0].printInterval(histoPW,sepString,i);
				for (int k=0;k<hists.length;k++) { hists[k].printBinSum(histoPW,sepString,i); }
				if (i<hists[0].infoS.length) { histoPW.print(hists[0].infoS[i]); }
				histoPW.println("");
			}
			talk ("wrote file for " + String.valueOf(hists.length) + " histograms with " + String.valueOf(hists[0].ptCt) + " pts");
		 	talk (" and " + String.valueOf(hists[0].passCt) + " passes"); 
			talkln ("");
			
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	
	
	public static void mkHistogramFile (Histogram [] histos, String path, String name) {
		try { 
			File histoFile = new File (path + "/" + name);
			FileWriter histoFW = new FileWriter(histoFile);
			PrintWriter histoPW = new PrintWriter(histoFW, true);
			// ** print parameter info **
			histoPW.println(name);
			histoPW.println(histos[0].pass1Params);
			histoPW.println(histos[0].pass2Params);
			histoPW.println(histos[0].pass3Params);
			// ** print column heading **
			int totPtCt = 0;
			histoPW.print("interval");
			for (int i=0;i<histos.length;i++) { 
				histoPW.print("; count=" + String.valueOf(histos[i].ptCt)); 
				totPtCt+=histos[i].ptCt;
			}
			histoPW.println(";total="+String.valueOf(totPtCt));
			// ** print data **
			for (int i=0;i<histos[0].ctVals.length; i++) {
				int totCtVal = 0;
				histoPW.print(String.valueOf(histos[0].intervalVals[i]));
				for (int j=0;j<histos.length;j++) { 
					histoPW.print(";" + String.valueOf(histos[j].ctVals[i])); 
					totCtVal += histos[j].ctVals[i];
				}
				histoPW.println(";" + String.valueOf(totCtVal));
			}
			talkln ("wrote histogram file with " + String.valueOf(totPtCt) + " pts");
		} catch (IOException ioe) { talkln ("Error creating a histogram file"); }
	}
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.println(info);
	}
	
}		
