package rocketBugsRender;
/*
	PlotFrame.... a jframe to plot in
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import java.io.*;
import java.text.*;

public class PlotFrame extends JFrame implements ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	RocketBugsRender MP;		// a reference back to the parent JFrame of this panel
	PlotPanel plotPanel;
	static final int maxSets = 15;
	JPanel xPanel = new JPanel();
	JPanel yPanel = new JPanel();
	JPopupMenu fileMenu = new JPopupMenu();
	JFileChooser opener = new JFileChooser();
	File [] dataFiles = new File [maxSets];
	DataSet analysisSet = new DataSet(0,"Analyzed");
	DataSet [] dataSets = new DataSet [maxSets];
	DataColumn xColumn = null;		// the x-axis values
	Pix2DColumn [] setPix = null;	// pointers reused in plotting
	AxisSet setAxes = new AxisSet();
	Axis [] allYAxes = null;
	int dataCt = 0;
	static Dimension plotDim = new Dimension(500,500);
	static Dimension frameDim = new Dimension(600,600);
	boolean plottedOnce = false;
	boolean scrollStop = true;
	boolean showScreenAve = true;
	
	// frame for file/variable control
	JButton plotButton;
	JFrame controlFrame = new JFrame();
	JMenuBar dataBar = new JMenuBar();
	JTabbedPane controlTab = new JTabbedPane();
	JPanel buttonPanel;
	ButtonTable holdYPanel = new ButtonTable(maxSets,false);
	ButtonTable yAxisPanel = new ButtonTable(maxSets,false);
	ButtonTable xAxisPanel = new ButtonTable(maxSets,true);	// make this mutually exclusive
	ButtonTable colorPanel = new ButtonTable(maxSets,false);
	ButtonTable anPanel = new ButtonTable(maxSets,true);
	JScrollBar runBar;
	JColorChooser colorChooser = new JColorChooser();
	JTextField positionInfo;
	
	// variable for mouse double-click
	static long lastClickTime = 0;
	
	// axis info
	int centerRow;
	int curExtent;
	int [] rowRange = new int [2];
	int [] maxRange = new int [2];
	int xZoomInc = 10;
	int yZoomInc = 10;
		
	
	public PlotFrame (RocketBugsRender MP) {
		this.MP = MP;
		setTitle (" !!! PLOT !!! ");
		setSize(frameDim);
		setResizable(true);
		setLocation(50,0);
		
		// make panels
		plotPanel = new PlotPanel(MP);
		//plotPanel.setToolTipText("PlotPanel");
		makePopUp();
		makeXPanel();
		makeYPanel();
		makeControlFrame();
		
		
		
		addKeyListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		addWindowListener(this);
		
		setBackColor(Color.black);
		initialize();
	}
	
	public void figureSizes () {
		frameDim = getSize();
		Dimension yPanelDim = yPanel.getPreferredSize();
		Dimension xPanelDim = xPanel.getPreferredSize();
		int plotX = frameDim.width - 2; //yPanelDim.width-2;
		int plotY = frameDim.height - xPanelDim.height -24;
		plotDim.width = plotX;
		plotDim.height = plotY;
	}
	
	public void initialize () {
		setVisible(true);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (plotPanel, BorderLayout.CENTER);
		getContentPane().add (xPanel, BorderLayout.SOUTH);
		//getContentPane().add (yPanel, BorderLayout.WEST);
		setVisible(true);
		figureSizes();
		plotPanel.initialize(plotDim);
		setVisible(true);
		requestFocus();
		startPosInGraphUpdateTimer();
	}
	
	public void posInGraphUpdate () {
		Point mousePt = MouseInfo.getPointerInfo().getLocation();
		mousePt.x -= this.getLocation().x;
		mousePt.y -= this.getLocation().y;
		String xStr = plotPanel.getXStringFromPixel(mousePt, xColumn);
		if (xStr != null) {
			positionInfo.setText("  x-value is " + xStr);
		} else {
			positionInfo.setText("  x-value is n/a");
		}
	}
	
	public void startPosInGraphUpdateTimer() {
		int delay = 100; //milliseconds
		ActionListener taskPerformer = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				posInGraphUpdate();
		    }
		};
		new Timer(delay, taskPerformer).start();
	}
	
	public boolean okToPlot () {
		if (dataCt == 0) { return false; }
		if (xColumn == null) { return false; }
		return true;
	}
	
	
	public void setRunBar () {
		runBar.setMaximum(maxRange[1]);
		runBar.setMinimum(maxRange[0]);
		curExtent = rowRange[1]-rowRange[0];
		runBar.getModel().setExtent(curExtent);
		centerRow = (rowRange[1]+rowRange[0])/2 - (rowRange[1]-rowRange[0])/2;
		if (centerRow < 0) { centerRow = 0; }
		runBar.setValue(centerRow);
	}
	
	public void setZoomInc() {
		xZoomInc = (int)(0.2*(rowRange[1]-rowRange[0]));
	}
	
	public void loadDataSet() {
		File toLoad;
		opener.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		int fileQuery = opener.showOpenDialog(this);
		if (fileQuery == JFileChooser.APPROVE_OPTION) {
            toLoad = opener.getSelectedFile();
            talkln ("Opening: " + toLoad.getName());
        } else {
            talkln ("Open command cancelled by user");
            return;
        }
		
		// check if directory or single file
		if (toLoad.isDirectory()) {
			File [] setsToLoad = toLoad.listFiles();
			for (int i=0;i<setsToLoad.length;i++) {
				if (!setsToLoad[i].getName().startsWith(".")) {	// hidden files ignored
					dataFiles[dataCt] = setsToLoad[i];
					DataSet newSet = new DataSet (dataFiles[dataCt].getAbsolutePath());
					addDataSet(newSet);
				}
			}
		} else {
			dataFiles[dataCt] = toLoad;
			DataSet newSet = new DataSet (dataFiles[dataCt].getAbsolutePath());
			addDataSet(newSet);
		}
	}
	
	public void addDataSet(DataSet newSet) {
		dataSets[dataCt] = newSet;
       	newSetUpdateGUI(newSet);
        dataCt++;
    }
	
	public void updateFieldsFromGUI() {
		for (int i=0;i<dataCt;i++) {
			dataSets[i].updateFromGUI();
		}
	}
	
	public void setXLimits () {
		XLimits xLimWindow = new XLimits (this);
	}
	
	public void setAllScales () {
		for (int i=0;i<dataCt;i++) {
			dataSets[i].scaleFields(plotPanel.lineDim, rowRange);
		}
	}
	
	public void setAllPixs() {
		for (int i=0; i<dataCt; i++) {
			dataSets[i].makePixTable();
		}
	}
	
	public void transRowRange () {
		int newCenter = runBar.getValue() + curExtent;
		int delta = newCenter - centerRow;
		rowRange[1]+= delta;
		if (rowRange[1] > maxRange[1]) { rowRange[1] = maxRange[1]; }
		rowRange[0] = rowRange[1]-curExtent;
		if (rowRange[0] < 0) { rowRange[0] = 0; }
		centerRow = newCenter;
		plotAll();
	}
	
	public void scrollRunForward () {
		int scrollStep = 10;
		for (int i=0;i<1;i++) {
		//while (!scrollStop) {
			int oldVal = rowRange[1];
			rowRange[1]+= scrollStep;
			if (rowRange[1] > maxRange[1]) {
				rowRange[1] = maxRange[1];
				scrollStop = true;
				scrollStep = rowRange[1]-oldVal;
			}
			rowRange[0]+= scrollStep;
			plotAll();
		}
	}
	
	public void scrollRunBack () {
		int scrollStep = -10;
		for (int i=0;i<1;i++) {
		//while (!scrollStop) {
			int oldVal = rowRange[0];
			rowRange[0]+= scrollStep;
			if (rowRange[0] < maxRange[0]) {
				rowRange[0] = maxRange[0];
				scrollStop = true;
				scrollStep = rowRange[0]-oldVal;
			}
			rowRange[1]+= scrollStep;
			plotAll();
		}
	}
	
	public DataColumn getXColumn () {
		updateFieldsFromGUI();
		// find x-axis data column
		for (int i=0;i<dataCt;i++) {
			for (int j=0;j<dataSets[i].dims[0];j++) {
				if (dataSets[i].dataTable[j].onX) {
					//talkln ("x-axis is now " + dataSets[i].dataTable[j].name);
					return dataSets[i].dataTable[j];
				}
			}
		}
		talkln ("No X-AXIS Chosen!");
		return null;
	}
	
	public DataColumn getAnColumn () {
		updateFieldsFromGUI();
		// find analysis data column
		for (int i=0;i<dataCt;i++) {
			for (int j=0;j<dataSets[i].dims[0];j++) {
				if (dataSets[i].dataTable[j].analyze) {
					//talkln ("analysis column is now " + dataSets[i].dataTable[j].name);
					return dataSets[i].dataTable[j];
				}
			}
		}
		talkln ("No Analysis Column Chosen!");
		return null;
	}
	
	public void ckXColumn () {
		DataColumn oldX = xColumn;
		DataColumn newX = getXColumn();
		if (newX != null) {
			if (newX.equals(oldX)) {
				return;		//do nothing
			} else {
				xColumn = newX;
				setMaxRowRange();
			}
		}
			
	}	
	
	public void setMaxRowRange () {
		maxRange[0] = 0;
		maxRange[1] = xColumn.rows;
		rowRange[0] = maxRange[0];
		rowRange[1] = maxRange[1];
		
	}
	
	private void xZoom (int delta) {
		rowRange[0]+= delta;
		rowRange[1]+= -delta;
		// check if out of bounds or crossing
		if (rowRange[1] < rowRange[0]) {
			int diff = rowRange[0]-rowRange[1];
			int fixDiff = (int)(diff/2-1);
			int temp = rowRange[1];
			rowRange[1] = rowRange[0]-fixDiff;
			rowRange[0] = temp+fixDiff;
		}
		if (rowRange[0] < 0) { rowRange[0] = 0; }
		setRunBar();
		setZoomInc();
		plotAll();
	}
	
	private void yZoom (int delta) {
	}
	
	private void xIn () { xZoom(xZoomInc); }
	private void xOut () { xZoom(-xZoomInc); }
	
	private void yIn () { yZoom(yZoomInc); }
	private void yOut () { yZoom(-yZoomInc); }
	
	public void plotAllPixs() {
		plotPanel.clearImage();
		Pix2DColumn pixCol;
		for (int i=0; i<dataCt; i++) {
			setPix = dataSets[i].getAllPix();
			plotPanel.plotPixs(setPix);
		}
		if (showScreenAve) {
			for (int i=0; i<dataCt; i++) {
				dataSets[i].reportAves(xColumn.range,xColumn);
			}
		}
		// assemble all y axes
		for (int i=0; i<dataCt; i++) {
			setAxes.appendAxes(dataSets[i].getAllYAxes());
		}
		plotPanel.plotYAxis(setAxes);
		setAxes.clear();
		// ---------------------
		plotPanel.plotXAxis(xColumn.axis);
		plotPanel.showGraphBox();
		plotPanel.repaint();
	}
	
	public void resetPlot () {
		if (okToPlot()) {
			setMaxRowRange();
			setRunBar();
			setZoomInc();
			plotAll();
		}
	}
	
	public void plotAll () {
		updateFieldsFromGUI();
		ckXColumn();
		if (okToPlot()) {
			setAllScales();
			setAllPixs();
			if (plotPanel.epsOut) { plotPanel.openEpsFile(); }
			plotAllPixs();
			if (plotPanel.epsOut) { plotPanel.closeEpsFile(); }
		}
	}
			
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Add Dataset")) {
			loadDataSet ();
		}
		
		if (act.equals("Plot")) {
			plotAll();
		}
		
		if (act.equals("Reset Plot")) { resetPlot(); }
		
		if (act.equals("EPS Grab")) {
			plotPanel.epsOut = true;
			plotAll();
		}
		
		if (act.equals("XLimits")) {
			setXLimits ();
		}
		
		if (act.equals("x+")) { xIn(); }
		if (act.equals("x-")) { xOut(); }
		if (act.equals("y+")) { yIn(); }
		if (act.equals("y-")) { yOut(); }
		
		if (act.equals("tighterAxis")) { plotPanel.labelSpacing += -2; }
		if (act.equals("looserAxis")) { plotPanel.labelSpacing += 2; }
		
		if (act.equals("flattenAxis")) {
			if (plotPanel.flatAxis) { 
				plotPanel.flatAxis = false;
				plotAll(); 
			} else { 
				plotPanel.flatAxis = true; 
				plotAll();
			}
		}
		
		if (act.equals("scrollForward")) {
			scrollStop = false;
			scrollRunForward();
		}
		
		if (act.equals("scrollBack")) {
			scrollStop = false;
			scrollRunBack();
		}
		
		if (act.equals("scrollStop")) {
			scrollStop = true;
		}
		
		
		if (act.equals("AnSteps")) {
			if (xColumn == null) { xColumn = getXColumn(); }	// if null try to get column
			if (xColumn == null) { talkln ("Choose and X-Axis!"); return; }	// if still null exit with message
			DataColumn inCol = getAnColumn();
			StepAnalyzer stepA = new StepAnalyzer(inCol, this, xColumn,false);
			DataColumn anC = stepA.getOutColumn();
			analysisSet.addCol(anC);
			if (analysisSet.dims[0] == 1) {
				addDataSet(analysisSet);
			} else {
				ammendSetUpdateGUI(analysisSet);
			}
		}
		
		if (act.equals("x-y-z")) {
			if (xColumn == null) { xColumn = getXColumn(); }	// if null try to get column
			if (xColumn == null) { talkln ("Identify data to work with by selecting x-column!"); return; }	// if still null exit with message
			DataColumn xC = xColumn.set.dataTable[1];
			DataColumn yC = xColumn.set.dataTable[2];
			DataColumn zC = xColumn.set.dataTable[3];
			DataColumn xyzC = new DataColumn(xColumn.rows,xColumn.set.name + ":x-y-z D", null);
			for (int i=0;i<xColumn.rows;i++) {
				xyzC.data[i] = Math.sqrt(xC.data[i]*xC.data[i] + yC.data[i]*yC.data[i] + zC.data[i]*zC.data[i]);
			}
			analysisSet.addCol(xyzC);
			if (analysisSet.dims[0] == 1) {
				addDataSet(analysisSet);
			} else {
				ammendSetUpdateGUI(analysisSet);
			}
		}
			
		
		if (act.equals("BackColor")) {
			Color newBack = JColorChooser.showDialog(colorChooser, "Background Color", plotPanel.backColor);
			if (newBack != null) {
				setBackColor(newBack);
				plotAll();
			}
		}
		
		if (act.equals("PlotColor")) {
			Object oe = e.getSource();
			AbstractButton ab = (AbstractButton)oe;  // cast source to abstract button since it must be
			choosePlotColor(ab);
		}
			
		
	}
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
		if (runBar.getValueIsAdjusting()) { transRowRange (); }
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
		talkln ("act = " + act);
		if (act.equals("P")) {
			scrollStop = false;
			scrollRunForward();
		}
		
		if (act.equals("O")) {
			scrollStop = true;
		}
		
		if (act.equals("I")) {
			scrollStop = false;
			scrollRunBack();
		}
		
		if (act.equals("V")) {
			if (showScreenAve) { showScreenAve = false; } else { showScreenAve = true; }
		}
	}
	
	public void mouseClicked (MouseEvent e) {
		long curClickTime = e.getWhen();
		long clickTime = curClickTime - lastClickTime;
		lastClickTime = curClickTime;		// 
		if (clickTime < 500) {
			//talkln ("double-clicked");
			int xPos = e.getX();
			int yPos = e.getY();
			// figure out which data column we're working on...
			Axis adjAx = null;
			int ckYAxes = plotPanel.curYAxes.length-1;
			while ((ckYAxes >=0) & (adjAx == null)) {
				if (xPos < plotPanel.offsets[0] - plotPanel.yAxesPositions[ckYAxes]) { adjAx = plotPanel.curYAxes[ckYAxes]; }
				ckYAxes += -1;
			}
			ckYAxes += 1;	// negate last decrement... now points to adjAx
			if (adjAx == null) { 
				talkln ("Never got an axis to adjust"); 
				return; 
			} else {
				talkln ("Axis is " + adjAx.parentDataCol.name);
				//adjAx.changePlotType();
				//plotAll();
				adjAx.parentDataCol.pProps.getProps(plotButton);
			}
		}
		
	}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
		
		if (e.isPopupTrigger()) {
			fileMenu.show(this,xPos,yPos);
			return;
		}
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
		int xPad = 30;	// pixel padding
		int yPad = 50;	// pixel padding
		if ((xPos > plotPanel.offsets[0] - xPad) & (xPos < plotPanel.offsets[0] + xPad)) {
			plotPanel.eraseGraphBox();
			int deltaX = xPos-plotPanel.offsets[0];
			plotPanel.offsets[0] += deltaX;
			plotPanel.lineDim.width -= deltaX;
			plotPanel.showGraphBox();
			plotPanel.repaint();
		} 
		
		if ((xPos < plotPanel.offsets[0] - xPad) & (plotPanel.curYAxes != null)) { // check if adjusting pixel height fraction or offset
			// figure out which data column we're working on...
			Axis adjAx = null;
			int ckYAxes = plotPanel.curYAxes.length-1;
			while ((ckYAxes >=0) & (adjAx == null)) {
				if (xPos < plotPanel.offsets[0] - plotPanel.yAxesPositions[ckYAxes]) { adjAx = plotPanel.curYAxes[ckYAxes]; }
				ckYAxes += -1;
			}
			ckYAxes += 1;	// negate last decrement... now points to adjAx
			if (adjAx == null) { talkln ("Never got an axis to adjust"); return; }
			if (yPos < plotPanel.plotDim.height - plotPanel.offsets[3] - adjAx.pProps.pixOrigin) {	// adjusting height fraction
				plotPanel.eraseYAxis(adjAx, plotPanel.yAxesPositions[ckYAxes]);
				double maxHeightFrac = (double)(plotPanel.lineDim.height-adjAx.pProps.pixOrigin)/(double)(plotPanel.lineDim.height);
				double newHeightFrac = 1 - (double)(yPos-plotPanel.offsets[2]-20)/(double)plotPanel.lineDim.height;
				if (newHeightFrac > maxHeightFrac) { newHeightFrac = maxHeightFrac; }
				if (newHeightFrac < 0.05) { newHeightFrac = 0.05; }
				adjAx.pProps.adjustPixFrac(newHeightFrac);
				adjAx.parentDataCol.setYAxis();
				plotPanel.plotYAxis(adjAx, plotPanel.yAxesPositions[ckYAxes]);
				plotPanel.repaint();
			} else {	// adjusting offset
				int maxOrigin = plotPanel.lineDim.height - adjAx.pixRange;
				int newPixOrigin = plotPanel.plotDim.height - plotPanel.offsets[3] + yPad  - yPos;
				if (newPixOrigin > maxOrigin) { newPixOrigin = maxOrigin; }
				if (newPixOrigin < 0) { newPixOrigin = 0; }
				plotPanel.eraseYAxis(adjAx, plotPanel.yAxesPositions[ckYAxes]);
				adjAx.pProps.adjustPixOrigin(newPixOrigin);
				adjAx.parentDataCol.setYAxis();
				plotPanel.plotYAxis(adjAx, plotPanel.yAxesPositions[ckYAxes]);
				plotPanel.repaint();
			}
		}

	}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {
		controlFrame.requestFocus();
		this.requestFocus();
		}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {
		talkln ("control frame size is " + String.valueOf(controlFrame.getWidth()) + "," + String.valueOf(controlFrame.getHeight()));
		initialize();
		plotAll();
	}
	
	public String checkName (String nck) {
		int sInd = nck.length()-12;
		if (sInd > 0) {
			if (nck.substring(sInd,nck.length()).equals("ListeriaInfo")) {
				return nck.substring(0,sInd);
			}
		}
		return nck;
	}
	
	public void sizeControlFrame() {
		Dimension bPanelDim = buttonPanel.getPreferredSize();
		Dimension tabDim = controlTab.getPreferredSize();
		int width1 = bPanelDim.width;
		int width2 = tabDim.width;
		int height = (int)(1.3*bPanelDim.height) + tabDim.height;
		if (width1 > width2) {
			controlFrame.setSize(width1,height);	
		} else {
			controlFrame.setSize(width2,height);	
		}
		controlFrame.show();
	}
		
	
	public void makeControlFrame() {
		// make subpanels
		makeDataBar();
		makeControlTab();
		// layout
		controlFrame.getContentPane().setLayout(new BorderLayout());
		controlFrame.getContentPane().add(dataBar,BorderLayout.NORTH);
		controlFrame.getContentPane().add(controlTab,BorderLayout.SOUTH);

		
		plotButton = new JButton ("Plot");
		plotButton.setFont(Env.controlFont);
		plotButton.setActionCommand("Plot");
		plotButton.setToolTipText("Plot/Replot");
		plotButton.addActionListener(this);
		plotButton.setSize(10,10);
		
		JButton addDataButton = new JButton ("Add Data");
		addDataButton.setFont(Env.controlFont);
		addDataButton.setActionCommand("Add Dataset");
		addDataButton.setToolTipText("Select data file to add");
		addDataButton.addActionListener(this);
		addDataButton.setSize(10,10);
		
		JButton resetButton = new JButton ("Reset Plot");
		resetButton.setFont(Env.controlFont);
		resetButton.setActionCommand("Reset Plot");
		resetButton.setToolTipText("Reset scales on current plot");
		resetButton.addActionListener(this);
		
		JButton stepAn = new JButton ("Line Fit");
		stepAn.setFont(Env.controlFont);
		stepAn.setActionCommand("AnSteps");
		stepAn.setToolTipText("Do a steps analysis");
		stepAn.addActionListener(this);
		
		JButton backColor = new JButton ("Back Color");
		backColor.setFont(Env.controlFont);
		backColor.setActionCommand("BackColor");
		backColor.setToolTipText("Change the plot background color");
		backColor.addActionListener(this);
		
		JButton epsGrab = new JButton ("EPS Grab");
		epsGrab.setFont(Env.controlFont);
		epsGrab.setActionCommand("EPS Grab");
		epsGrab.setToolTipText("Grab an EPS file of the current plot");
		epsGrab.addActionListener(this);
		
		JButton setLims = new JButton ("X Limits");
		setLims.setFont(Env.controlFont);
		setLims.setActionCommand("XLimits");
		setLims.setToolTipText("Manually set X-axis limits");
		setLims.addActionListener(this);
		
		JButton blank = new JButton ("x-y-z D");
		blank.setFont(Env.controlFont);
		blank.setActionCommand("x-y-z");
		blank.setToolTipText("Create column from sqrt(x2+y2+z2)");
		blank.addActionListener(this);
		
		/*JButton batchHist = new JButton ("Batch Histo");
		batchHist.setFont(Env.controlFont);
		batchHist.setActionCommand("BatchHist");
		batchHist.setToolTipText("Create histogram from multiple run files");
		batchHist.addActionListener(this);
		
		JButton pairWise = new JButton ("Pairwise");
		pairWise.setFont(Env.controlFont);
		pairWise.setActionCommand("PairWise");
		pairWise.setToolTipText("Create histogram pair-wise displacements");
		pairWise.addActionListener(this);*/
		
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(2,4));
		buttonPanel.add(plotButton);
		buttonPanel.add(resetButton);
		buttonPanel.add(addDataButton);
		buttonPanel.add(stepAn);
		buttonPanel.add(backColor);
		buttonPanel.add(epsGrab);
		buttonPanel.add(setLims);
		buttonPanel.add(blank);
		//buttonPanel.add(batchHist);
		//buttonPanel.add(pairWise);
		controlFrame.getContentPane().add(buttonPanel,BorderLayout.CENTER);
		
		
		// location and attributes
		Point loc = this.getLocation();
		controlFrame.setLocation(loc.x + this.getWidth()+10, loc.y);
		controlFrame.setResizable(false);
		sizeControlFrame();
	}
	
	public void makePopUp () {
		fileMenu = new JPopupMenu();
		fileMenu.setInvoker(this);
		fileMenu.add ("**File Options**");
		JMenuItem plotVars = new JMenuItem("Add Dataset");
		plotVars.setActionCommand("Add Dataset");
		plotVars.addActionListener(this);
		JMenuItem op2 = new JMenuItem("something else");
		op2.setActionCommand("op2");
		op2.addActionListener(this);
		JMenuItem op3 = new JMenuItem("something else");
		op3.setActionCommand("op3");
		op3.addActionListener(this);
		fileMenu.add(plotVars);
		fileMenu.add(op2);
		fileMenu.add(op3);
	}
	
	public void makeDataBar () {
		
	}
	
	public void choosePlotColor (AbstractButton colB) {
		String title = "Choose color for " + colB.getText();
		Color oldColor = colB.getForeground();
		Color newColor = JColorChooser.showDialog(colorChooser, title, oldColor);
		if (newColor != null) {
			colB.setForeground(newColor);
		}
		colB.setSelected(false);
	}
	
	public void setBackColor (Color newColor) {
		plotPanel.backColor = newColor;
		colorPanel.setBackground(newColor);
		colorPanel.updateColors(newColor);
		xPanel.setBackground(newColor);
		yPanel.setBackground(newColor);
	}
		
	
	public void newSetUpdateGUI (DataSet newSet) {
		xAxisPanel.addList(newSet.xAxisList);
		yAxisPanel.addList(newSet.yAxisList);
		holdYPanel.addList(newSet.holdYList);
		colorPanel.addList(newSet.colorList,"PlotColor",this);
		anPanel.addList(newSet.anList);
		sizeControlFrame();
	}
	
	public void removeSetUpdateGUI (DataSet rmSet) {
		xAxisPanel.rmList(rmSet.xAxisList);
		yAxisPanel.rmList(rmSet.yAxisList);
		holdYPanel.rmList(rmSet.holdYList);
		colorPanel.rmList(rmSet.colorList);
		anPanel.rmList(rmSet.anList);
		sizeControlFrame();
	}
	
	public void ammendSetUpdateGUI (DataSet amSet) {
		xAxisPanel.ammendList(amSet.xAxisList);
		yAxisPanel.ammendList(amSet.yAxisList);
		holdYPanel.ammendList(amSet.holdYList);
		colorPanel.ammendList(amSet.colorList,"PlotColor",this);
		anPanel.ammendList(amSet.anList);
		sizeControlFrame();
	}
	
	
	public void makeControlTab () {
		controlTab.setFont(Env.controlFont);
		controlTab.addTab("X axis",xAxisPanel);
		controlTab.addTab("Y axis",yAxisPanel);
		controlTab.addTab("Hold Y",holdYPanel);
		controlTab.addTab("Colors",colorPanel);
		controlTab.addTab("Analyze",anPanel);
		controlFrame.getContentPane().add(controlTab,BorderLayout.SOUTH);
	}
	
	
	public void makeXPanel () {
		runBar = new JScrollBar(JScrollBar.HORIZONTAL);
		runBar.addAdjustmentListener(this);
		
		Dimension zoomSize = new Dimension (45,20);
		JButton inButton = new JButton ("+");
		inButton.setActionCommand("x+");
		inButton.setToolTipText("Zoom In");
		inButton.addActionListener(this);
		inButton.setPreferredSize(zoomSize);
		inButton.setBackground(Color.black);
		inButton.setForeground(Color.black);
		
		JButton outButton = new JButton ("-");
		outButton.setActionCommand("x-");
		outButton.setToolTipText("Zoom Out");
		outButton.addActionListener(this);
		outButton.setPreferredSize(zoomSize);
		outButton.setBackground(Color.black);
		outButton.setForeground(Color.black);
		
		/*JButton tighterB = new JButton ("tight");
		tighterB.setFont(Env.controlFont);
		tighterB.setActionCommand("tighterAxis");
		tighterB.setToolTipText("Move axes together");
		tighterB.addActionListener(this);
		tighterB.setBackground(Color.black);
		tighterB.setForeground(Color.black);
		
		JButton looserB = new JButton ("loose");
		looserB.setFont(Env.controlFont);
		looserB.setActionCommand("looserAxis");
		looserB.setToolTipText("Move axes apart");
		looserB.addActionListener(this);
		looserB.setBackground(Color.black);
		looserB.setForeground(Color.black);*/
		
		JButton flattenB = new JButton("flat");
		flattenB.setFont(Env.controlFont);
		flattenB.setActionCommand("flattenAxis");
		flattenB.setToolTipText("all y-axis in one column");
		flattenB.addActionListener(this);
		flattenB.setBackground(Color.black);
		flattenB.setForeground(Color.black);
		
		JButton scrollFB = new JButton(">>");
		scrollFB.setFont(Env.controlFont);
		scrollFB.setActionCommand("scrollForward");
		scrollFB.setToolTipText("scroll forward");
		scrollFB.addActionListener(this);
		scrollFB.setBackground(Color.black);
		scrollFB.setForeground(Color.black);
		
		JButton scrollBB = new JButton("<<");
		scrollBB.setFont(Env.controlFont);
		scrollBB.setActionCommand("scrollBack");
		scrollBB.setToolTipText("scroll backward");
		scrollBB.addActionListener(this);
		scrollBB.setBackground(Color.black);
		scrollBB.setForeground(Color.black);
		
		JButton scrollStopB = new JButton("stop");
		scrollStopB.setFont(Env.controlFont);
		scrollStopB.setActionCommand("scrollStop");
		scrollStopB.setToolTipText("stop scrolling");
		scrollStopB.addActionListener(this);
		scrollStopB.setBackground(Color.black);
		scrollStopB.setForeground(Color.black);
		
		
		/*Box xZBox = new Box(BoxLayout.X_AXIS);
		xZBox.add(inButton);
		xZBox.add(outButton);
		xZBox.add(Box.createHorizontalStrut(20));*/
		JPanel butPan = new JPanel(new GridLayout(2,3));
		butPan.add(inButton);
		butPan.add(outButton);
		butPan.add(flattenB);
		butPan.add(scrollBB);
		butPan.add(scrollStopB);
		butPan.add(scrollFB);
		
		JPanel infoPan = new JPanel();
		infoPan.setBackground(plotPanel.backColor);
		positionInfo = new JTextField("  x-value is n/a");
		positionInfo.setEditable(false);
		//positionInfo.setSize(200,40);
		positionInfo.setColumns(15);
		//positionInfo.setFont(Env.controlFont);
		positionInfo.setBackground(plotPanel.backColor);
		positionInfo.setForeground(plotPanel.boxColor);
		positionInfo.setBorder(BorderFactory.createEmptyBorder());
		infoPan.add(positionInfo);
		
		xPanel.setLayout(new BorderLayout());
		xPanel.add (infoPan, BorderLayout.WEST);
		xPanel.add(butPan, BorderLayout.EAST);
		xPanel.add (runBar, BorderLayout.SOUTH);
		xPanel.setBackground(Color.black);
	}
	
	public void makeYPanel () {
		Font zoomFont = new Font(null,Font.BOLD,8);
		Dimension zoomSize = new Dimension (45,40);
		JButton inButton = new JButton ("+");
		inButton.setFont(zoomFont);
		inButton.setActionCommand("y+");
		inButton.setToolTipText("Zoom In");
		inButton.addActionListener(this);
		inButton.setPreferredSize(zoomSize);
		
		JButton outButton = new JButton ("-");
		outButton.setFont(zoomFont);
		outButton.setActionCommand("y-");
		outButton.setToolTipText("Zoom Out");
		outButton.addActionListener(this);
		outButton.setPreferredSize(zoomSize);
		outButton.setBackground(Color.black);
		outButton.setForeground(Color.black);
		
		Box yZBox = new Box(BoxLayout.Y_AXIS);
		yZBox.add(outButton);
		yZBox.add(inButton);
		
		yPanel.setLayout(new BorderLayout());
		yPanel.add (yZBox, BorderLayout.NORTH);
		yPanel.setBackground(Color.black);
	}
	
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}

	
	
	
	
	
	
	
	
}